import { ChildProperty } from '@syncfusion/ej2-base';
import { SheetModel, CellModel } from './index';
import { CellStyleModel, HyperlinkModel } from '../common/index';
import { RowModel } from './row-model';
/**
 * Represents the cell.
 */
export declare class Cell extends ChildProperty<RowModel> {
    /**
     * Defines the value of the cell which can be text or number.
     * @default ''
     */
    value: string;
    /**
     * Defines the formula or expression of the cell.
     * @default ''
     */
    formula: string;
    /**
     * Specifies the index of the cell.
     * @default 0
     * @asptype int
     */
    index: number;
    /**
     * Specifies the number format code to display value in specified number format.
     * @default 'General'
     */
    format: string;
    /**
     * Specifies the cell style options.
     *  ```html
     * <div id='Spreadsheet'></div>
     * ```
     * ```typescript
     * let spreadsheet: Spreadsheet = new Spreadsheet({
     *      sheets: [{
     *       ...
     *            rows: [{
     *                  cells: [{ value: '12', index: 2,  style: { fontWeight: 'bold', fontSize: 12, fontStyle: 'italic', textIndent: '2pt'
     *                         backgroundColor: '#4b5366', color: '#ffffff' } }]
     *                  }]
     *            }]
     *  });
     * spreadsheet.appendTo('#Spreadsheet');
     * ```
     * @default {}
     */
    style: CellStyleModel;
    /**
     * Specifies the hyperlink of the cell.
     * @default ''
     * @aspType SpreadsheetHyperlink
     */
    hyperlink: string | HyperlinkModel;
}
/**
 * @hidden
 */
export declare function getCell(rowIndex: number, colIndex: number, sheet: SheetModel, isInitRow?: boolean): CellModel;
/**
 * @hidden
 */
export declare function setCell(rowIndex: number, colIndex: number, sheet: SheetModel, cell: CellModel, isExtend?: boolean): void;
/**
 * @hidden
 */
export declare function getCellPosition(sheet: SheetModel, indexes: number[]): {
    top: number;
    left: number;
};
/** @hidden */
export declare function skipDefaultValue(style: CellStyleModel, defaultKey?: boolean): CellStyleModel;
